#! /bin/sh
# \
exec wish${TCL} "$0" ${1+"$@"}

#
# AitCS Example
#
# Using AitCS with a GUI
#
# Stuart Cassoff
# Fall 2020
#


#
# This example shows how using AitCS with a gui
# can make it easier to work with options like
# -textvariable or -command, where the global
# scope as well as any namespace scope
# sometimes needs to be considered.
#

#
# This example also shows how using AitCS with a gui
# can make it easier to work with widget paths
# since oftentimes many widgets are created
# but few are manipulated afterwards.
#
# Widget paths saved in an AitCS can then be easily
# accessed by name elswehere in the program.
# 


package require aitcs

namespace eval ::aitcs { namespace eval examples { namespace eval gui {

proc createGui {} {
	bind . <Control-q> { destroy . }

	# Use an AitCS to hold widgets needed elsewhere
	# Use other AitCS's as data sources for gui elements

	# Save widget paths in the 'gui' AitCS
	# A few widget paths are needed in other places in the program
	# We can use names instead of ugly widget paths
	#
	aitcs new gui

	# Copy the 'listDisabled and 'editReadonly' elements of the 'cfg' AitCS
	#
	foreach elem {listDisabled editReadonly} {
		gui $elem [cfg $elem]
	}

	# Use the 'what' element of the 'cfg' AitCS for the title
	#
	# The '&' (variable reference) operator is used to get the fqn of the AitCS array element
	#
	grid [label .l0 -textvariable [cfg& what] -font TkBoldFont] -columnspan 2 -pady {2 0}

	grid [frame .f0] [frame .f1] -padx 6 -pady 4

	# Save the listbox widget path in the 'anilist' element of the 'gui' AitCS
	#
	# For the -listvariable, use the '&' (variable reference) operator
	# to get the fqn of the 'names' element of the 'data' AitCS
	#
	grid [gui anilist \
		[listbox .f0.o0 \
			-listvariable [data& names] \
			-width 6 -height 5]] -sticky nsew

	# Use the 'fields' element of the 'cfg' AitCS for the field names
	#
	# Use the 'rec' AitCS for the -textvariables of the entries
	#
	# Save the widget paths of the entries in the 'gui' AitCS
	#
	foreach field [cfg fields] {
		grid [label .f1.l$field -text [string totitle $field]] \
		     [gui $field \
			[entry .f1.e$field -textvariable [rec& $field] -width 10]] \
		-sticky w
	}

	# Use the 'listDisabled' element of the 'gui' AitCS for the -variable
	#
	# The '&&' (command reference) operator is used to get the fqn of the AitCS command
	#
	grid [checkbutton .k0 \
		-variable [gui& listDisabled] \
		-onvalue disabled \
		-offvalue normal \
		-text "Disabled" \
		-command "\[[gui&&] anilist\] configure -state \[[gui&&] listDisabled\]"
	]

	# Use the 'editReadonly' element of the 'gui' AitCS for the -variable
	#
	grid [checkbutton .k1 \
		-variable [gui& editReadonly] \
		-onvalue readonly \
		-offvalue normal \
		-text "Readonly" \
		-command [namespace current]::toggleReadonly \
	] -row 2 -column 1 -sticky e

	grid [label .l1 -text "Control-q to exit"] -columnspan 2 -pady {8 2}

	# The 'anilist' element of the 'gui' AitCS contains
	# the widget path of the listbox (set above)
	#
	bind [gui anilist] <<ListboxSelect>> [namespace current]::listboxSelect
}


proc listboxSelect {} {
	if {[set sel [lindex [[gui anilist] curselection] 0]] eq {}} { return }
	set animal [[gui anilist] get $sel]

	# data->rec
	#
	# Use the '->' (structure dereference) operator
	# to access the animal's info
	#
	foreach field [cfg fields] {
		rec $field [data-> $animal $field]
	}
}


proc toggleReadonly {} {
	# Use the 'fields' element of the 'cfg' AitCS for the field names
	# Use the 'gui' AitCS to obtain the widget paths
	# Use the 'editReadonly' element of the 'gui' AitCS (set by the checkbutton)
	#
	foreach field [cfg fields] {
		[gui $field] configure -state [gui editReadonly]
	}
}


proc initData {} {
	# Create an AitCS to hold the names of the data elements
	# and references to AitCS's used as data elements
	#
	# Use the '->' (struct dereference) operator in order to
	# demonstrate usage of the '->' (struct dereference) operator
	#
	aitcs new data

	# The animals in the menagerie
	#
	data names [list dog cat cow pig]

	# Create each animal as an AitCS and supply default values
	# Each AitCS will have a random name but they will be accessed
	# via the 'data' AitCS using the '->' (structure dereference) operator
	#
	# Create the first three animals, supplying initial values
	#
	data dog [aitcs new -- type Dog name Mika   colour Black greeting Woof]
	data cat [aitcs new -- type Cat name Fluffy colour White greeting Meow]
	data cow [aitcs new -- type Cow name Bossie colour Brown greeting Moo]

	# Create this animal in two steps
	#
	# Create
	# Here, the 'pig' element of the 'data' AitCS is set to
	# 'peeg', the name of the newly-created AitCS that
	# will be used for the 'pig' animal.
	#
	# This will work providing the AitCS commands
	# for 'data' and 'peeg' are in the same namespace.
	# Better would be to use [peeg&&] which provides
	# the fully qualified name.
	#
	aitcs new peeg
	data pig peeg
	#
	# Supply values
	# Reference the 'pig' AitCS via the 'pig' element
	# using the '->' (struct dereference) operator
	#
	data-> pig type Pig name Babe colour Pink greeting Oink
}


proc initRec {} {
	# This will hold the values of the current record

	# Create
	aitcs+ new rec

	# Set each field to an initial value
	#
	# The '=' (setto) operator will set
	# all specified elements to the last argument 
	#
	rec= {*}[cfg fields] ""
}


proc initCfg {} {
	# Create and set initial values for the 'cfg' AitCS
	#
	# Create command  Name  EndOpts   Initial values
	# v               v     v         v
	aitcs new         cfg   --        \
		what   Animals \
		fields [list type name colour greeting] \
		listDisabled  normal \
		editReadonly  normal
}


proc go {} {
	package require Tk
	namespace import ::aitcs::aitcs ::aitcs::aitcs+
	initCfg
	initRec
	initData
	createGui
	focus [gui anilist]
	# Select first item. Must be a better way.
	::tk::ListboxUpDown [gui anilist] 0
}


}}}; # ::aitcs::examples::gui


if {[info exists ::argv0] && [info script] eq $::argv0} {
	::aitcs::examples::gui::go {*}$::argv
}


# EOF
